import logging
import argparse
import configargparse
import sys

_config = None

def normalize_extensions(ext_list: list[str]) -> list[str]:
    # Normalizzo la lista estensioni
    # L'ordine è fondamentale
    default_ext=['.docx', '.doc', '.odt', '.txt', '.xlsx', '.pptx', '.pdf']

    if not ext_list:
        return default_ext

    return [ext if ext.startswith('.') else f'.{ext}' for ext in ext_list]


# Promemoria
# .debug() per info verbose
# .info() per operazioni principali riuscite
# .warning() per risultati problematici
# .error() per eccezioni
def configure_logging(debug_flag: bool, environment: str = None, log_file: str = None):
    # Livello globale
    logging.basicConfig(
        force=True,
        handlers=[]
    )
    root_logger = logging.getLogger()
    root_logger.setLevel(logging.WARNING)

    # log gtagger
    app_logger = logging.getLogger("gtagger")
    app_logger.setLevel(logging.DEBUG if debug_flag else logging.INFO)
    formatter = logging.Formatter("%(asctime)s [%(levelname)s] %(name)s %(message)s")

    # Crea e aggiungi handler per la console
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(formatter)
    app_logger.addHandler(console_handler)

    # Aggiungi handler per il file se specificato
    if environment == "development" and log_file:
        file_handler = logging.FileHandler(log_file)
        file_handler.setFormatter(formatter)
        app_logger.addHandler(file_handler)

    if debug_flag:
        import jurigged
        logging.getLogger("watchdog").setLevel(logging.INFO)
        jurigged.watch(pattern="*gtagger*.py")



def get_config() -> argparse.Namespace:
    global _config

    if _config is not None:
        return _config

    parser = configargparse.get_argument_parser(
        config_file_parser_class=configargparse.YAMLConfigFileParser,
        formatter_class=configargparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "-c", "--config", required=False, is_config_file=True, help="config file path"
    )

    parser.add_argument(
        "--config-save",
        required=False,
        is_write_out_config_file_arg=True,
        help="config file path",
    )

    parser.add_argument("files", nargs="*", default=[], help="list of documents")
    parser.add_argument("--debug", action="store_true", default=False, help="debug")
    parser.add_argument("--dry-run", action="store_true", default=False, help="Check only")
    parser.add_argument("--task", default="report", choices=["upload", "report", "ai", "convert", "setup", "patch","semantic-rebuild"], help="Desidered operation")
    parser.add_argument("--model", default="gemini-2.0-flash-lite-001", help="model")
    parser.add_argument("--person", action="append", default=["Federico Bricolo","Maurizio Danese","Adolfo Rebughini","Matteo Gelmetti","Romano Artoni"], help="list of important people")
    parser.add_argument("--env", choices=["development", "production"], default="development", help="Running mode(development o production)")
    parser.add_argument("-e", "--extension", action="append", help="list of valuable extensions")
    parser.add_argument("--log-file", default=None, help="Log file")
    parser.add_argument("--paperless", default="https://paper.csgalileo.org", help="Paperless URL")
    parser.add_argument("--tika", default="http://localhost:9998", help="Tika server")

    # pytest non va d'accordo con argomenti required=True
    if "pytest" in sys.modules:
        parser.add_argument("-t", "--token", default="1234", help="paperless API token")
    else:
        parser.add_argument("-t", "--token", required=True, help="paperless API token")

    _config, _ = parser.parse_known_args()


    configure_logging(_config.debug, _config.env, _config.log_file)
    _config.extension = normalize_extensions(_config.extension)

    return _config


