from abc import ABC, abstractmethod
import logging
from typing import Optional
from pathlib import Path
import os

import httpx

from .config import get_config


log = logging.getLogger(__name__)
config = get_config()


# Uniformo le eccezioni (mi vengono in mente queste due)
class ConversionError(Exception):
    """Eccezione di base per ogni errore di conversione"""
    pass

class FileNotFoundError(ConversionError):
    """RIl file che ho passato non esiste"""
    pass

class UnsupportedFormatError(ConversionError):
    """Il formato che voglio convertire non è supportato"""
    pass


class BaseConverter(ABC):
    """Classe astratta per ogni backend di conversione"""
    @abstractmethod
    def extract_text(self, source_path: Path, target_path: Optional[Path] = None) -> Optional[str]:
        """
        Estrae il testo da un file
        
        Args:
            filepath: Path del file
            target_path: Path opzionaòle per eventuale salvataggio
            
        Returns:
            Testo convertito in formato stringa oppure None se c'e' un errore
        """
        pass
    
    def _validate_input_file(self, filepath: Path) -> None:
        """Valida lo stato del file"""
        # Sto delegando alla classe la gestione del file
        if not filepath.exists():
            raise FileNotFoundError(f"File non trovato: {filepath}")
        if not filepath.is_file():
            raise ValueError(f"E' necessario un file, non un persorso: {filepath}")
        if not os.access(filepath, os.R_OK):
            raise PermissionError(f"Impossibile leggere il file: {filepath}")


# Implementazione Markitdown (dovrebbe essere sufficientemente robusta)
class MarkitdownConverter(BaseConverter):
    def __init__(self):
        self.supported_formats = config.extension  


    def extract_text(self, source_path: str, target_path: Optional[Path] = None) -> Optional[str]:
        """Estrazione tramite MarkitDown"""
        from markitdown import MarkItDown, FileConversionException, UnsupportedFormatException

        self._validate_input_file(source_path)
        if source_path.suffix.lower() not in self.supported_formats:
            raise UnsupportedFormatError(f"Formato non supportato: {source_path.suffix}")
        
        try:         
            log.info(f"Conversione del file {source_path} usando MarkItDown") 
            md = MarkItDown()       

            result = md.convert(source_path).text_content
            
            if target_path:
                target_path.write_text(result, encoding="utf-8")
                log.info(f"File convetito salvato in {target_path}")

            return result
        
        #Intercetto le  eccexioni di MarkItDown e le ripropongo come Eccezioni normallizzate
        except FileConversionException as e:
            log.error(f"La conversione del file {source_path} ha prodotto l'errore: {e}.")
            raise ConversionError(f"Conversione di {source_path} fallita") from e
        except UnsupportedFormatException as e:
            raise UnsupportedFormatError(f"Formato non supportato: {source_path}")
        except Exception as e:
            log.error(f"La conversione del file {source_path} ha prodotto un errore non gestito")
            raise ConversionError(f"Errore imprevisto nella conversione di {source_path}") from e


# Direi che il UTF-8 è quello che serve, altrimenti compaiono caratteri strani
class TikaConverter(BaseConverter):
    def __init__(self, base_url: str = config.tika, timeout: float = 30.0, default_encoding: str = "utf-8"):
        self.client = httpx.Client(
            base_url=base_url,
            timeout=timeout,
            headers={"Accept": f"text/plain; charset={default_encoding}"}
        )
        self.base_url = base_url
        self.default_encoding = default_encoding
        self.supported_formats = [".md"] +config.extension


    def get_language(self, source_path: Path, target_path: Optional[Path] = None, fix_mojibake: bool = True) -> str:
        # Questo è ridondante e andrebbe sistemato
        self._validate_input_file(source_path)
        if source_path.suffix.lower() not in self.supported_formats:
            raise UnsupportedFormatError(f"Formato non supportato: {source_path.suffix}")
        
        log.debug(f"Invio {source_path} a Tika")
        try:
            with open(source_path, "rb") as f:
                response = self.client.put(
                    "/language/string",
                    content=f.read(),
                    headers={"Accept-Charset": self.default_encoding}
                )
                response.raise_for_status()
                text = response.text

            # Uso strip() perché se il pdf non contiene testo, tika può restituire una stringa di soli spazi
            # ma io uso la lunghezza del file md nulla per sapere se è ocr
            text = text.strip()
            # if target_path:
            #     self._write_output(text, target_path)

            return text    
        except httpx.HTTPStatusError as e:
            log.error(f"HTTP error {e.response.status_code} da Tike: {e.response.text}")
            raise ConversionError(f"Errore server Tika: {e}") from e
        except httpx.RequestError as e:
            log.error(f"Connessione con il server Tika non riuscito: {e}")
            raise ConversionError(f"COllegamento al server Tika {self.base_url} fallito") from e
        except Exception as e:
            log.error(f"La conversione del file {source_path} ha prodotto un errore non gestito")
            raise ConversionError(f"Errore imprevisto nella conversione di {source_path}") from e
        

    def extract_text(self, source_path: Path, target_path: Optional[Path] = None, fix_mojibake: bool = True) -> str:
        # Questo è ridondante e andrebbe sistemato
        self._validate_input_file(source_path)
        if source_path.suffix.lower() not in self.supported_formats:
            raise UnsupportedFormatError(f"Formato non supportato: {source_path.suffix}")
        
        log.debug(f"Invio {source_path} a Tika")
        try:
            with open(source_path, "rb") as f:
                response = self.client.put(
                    "/tika",
                    content=f.read(),
                    headers={"Accept-Charset": self.default_encoding}
                )
                response.raise_for_status()
                text = response.text

            # Uso strip() perché se il pdf non contiene testo, tika può restituire una stringa di soli spazi
            # ma io uso la lunghezza del file md nulla per sapere se è ocr
            text = text.strip()
            if target_path:
                self._write_output(text, target_path)

            return text
        
        except httpx.HTTPStatusError as e:
            log.error(f"HTTP error {e.response.status_code} da Tike: {e.response.text}")
            raise ConversionError(f"Errore server Tika: {e}") from e
        except httpx.RequestError as e:
            log.error(f"Connessione con il server Tika non riuscito: {e}")
            raise ConversionError(f"COllegamento al server Tika {self.base_url} fallito") from e
        except Exception as e:
            log.error(f"La conversione del file {source_path} ha prodotto un errore non gestito")
            raise ConversionError(f"Errore imprevisto nella conversione di {source_path}") from e


    def _write_output(self, text: str, output_path: Path):
        """Scrive il risultato su file con gestione errori"""
        try:
            output_path.write_text(text, encoding=self.default_encoding)
            log.info(f"File scritto con successo in {output_path}")
        except IOError as e:
            log.error(f"Errore scrittura file: {str(e)}")
            raise ConversionError(f"Impossibile scrivere sul file {output_path}") from e


    # Supporta with ... as:
    def __enter__(self):
        return self


    def __exit__(self, exc_type, exc_val, exc_tb):
        self.client.close()


class Convert:
    """Classe che espone il backend di conversione e i metodi 'pubblici'"""
    def __init__(self, backend: str = "tika"):
        if backend == "tika":
            self._converter = TikaConverter()
        elif backend == "markitdown":
            self._converter = MarkitdownConverter()
        else:
            raise ValueError(f"Backend non supportato: {backend}")

    def extract_text(self, source_path: Path, target_path: Optional[Path] = None) -> str:
        return self._converter.extract_text(source_path, target_path)        