import datetime
from typing import Optional
import re
import os
import logging
import traceback

log = logging.getLogger(__name__)

class DateHelper:
    @staticmethod
    def is_acceptable_data(date_to_check: str) -> bool:
        """
        Controlla che la data passata come argomento sia un valido formato ISO YYYY-MM-DD

        Args:
            date_to_check (str): una stringsa che contiene la data da validare.

        Returns:
            True se la data è nel formato desiderato e non futura
            False in ogni altro caso

        """
        pattern = r"^\d{4}-\d{2}-\d{2}$"
        is_valid_date = re.match(pattern, date_to_check)
        if is_valid_date is not None:
            today = datetime.date.today()
            other_day = datetime.date.fromisoformat(date_to_check)
            log.debug(f"Valid date returned from ai: {date_to_check}")
            return other_day <= today
        return False

    @staticmethod
    def get_last_change(filename: str) -> Optional[str]:
        """
        Legge la data di modifica di un file

        Args:
            filename (str): la stringa contentente il path del file

        Returns:
            Optional[str]: la data in formato ISO (YYYY-MM-DD) o None in caso di errore

        Example:
            >>> parser = Tagger()
            >>> parser.get_last_change("/mnt/data/file.txt")
            '2023-10-30'    
        """
        try:
            t_modifica = os.path.getmtime(filename)
            return datetime.datetime.fromtimestamp(t_modifica).date().isoformat()
        except FileNotFoundError:
            log.error(f"file {filename} non trovato")
        except Exception as e:
            log.error(f"Errore generico: {e}")
            log.error(traceback.format_exc())
            
        return None

