import logging
import json
from typing import Optional, Dict, Any, Union

from langchain_core.messages import HumanMessage, SystemMessage
from .ai import Ai

log = logging.getLogger(__name__)

class LLMInvoker:
    def __init__(self):
        self._ai = None

    @property
    def ai(self):
        if self._ai is None:
            self._ai = Ai()
        return self._ai
    
    def analyze(self, prompt: str, 
                payload: str, 
                system_message: str = "Sei un sistema di indicizzazione per documenti.",
                template: Any=None) -> Union[str, object]:
        """
        Richiede un'analisi ad un LLM.

        L'analisi può essere di tipo struttura, se viene fornito una classe template oppure
        non strutturato.

        Args:
            json_string (str): La stringa contenente il JSON formattato con ```json ... ```.
            prompt (str): il prompt da fornire al motore LLM
            payload (str); il testo sul quale effettuare l'analisi
            system_message (str): lo scopo assegnato alla sessione
            template (Any): un template per dati strutturati

        Returns:
            str: la risposta ottenuta dal motore llm

        """        
 
        messages = [
            SystemMessage(prompt),
            HumanMessage(payload),
        ]        
        if template:
            return self.ai.structured_invoke(template, messages)
        else:
            response = self.ai.invoke(messages)
            data = self.parse_json(response.content)
            usage = getattr(response, 'usage_metadata', {})
            if not usage:
                log.warning("Nessun dato contabile relativo ai token.")
            return data, usage
        

    def parse_json(self, json_string: str) -> Dict[str, Any]:
        """
        Estrae e analizza un oggetto JSON racchiuso tra delimitatori ```

        Args:
            json_string (str): La stringa contenente il JSON formattato con ```json ... ```.
            
        Returns:
            Dict[str, Any]: Il dizionario risultante dal parsing del JSON.
        
        Raises:
            ValueError: Se il JSON non è valido o non viene trovato un blocco JSON corretto.

        Example:
            >>> parser = Tagger()
            >>> parser.parse_json("abcdef ```json {\"key\": \"value\"} ``` ghilmn")
            {'key': 'value'}    
            
        """   
        
        start_index = json_string.find("```json") + len("```json")
        end_index = json_string.find("```", start_index)
        
        if start_index == -1 or end_index == -1:
            raise ValueError("Blocco JSON non trovato o formattazione errata")

        extracted_json = json_string[start_index:end_index].strip()
        
        try:
            data = json.loads(extracted_json)
        except json.JSONDecodeError as e:
            raise ValueError(f"Errore nel parsing del JSON: {e}")

        return data