import os
from pathlib import Path
import json
import logging
from datetime import datetime
import sys
import tempfile

import ocrmypdf
from markitdown import MarkItDown, FileConversionException, UnsupportedFormatException

from gtagger.llm_invoker import LLMInvoker
from gtagger.date_helper import DateHelper
from gtagger.stats_collector import StatsCollector

from gtagger.convert import Convert, ConversionError,TikaConverter
from gtagger.model import Document, JsonItem, CustomField
from gtagger.tagger import Tagger
from gtagger.config import get_config
from gtagger.model import Document, CustomField, document_dict, parse_iso_date
from gtagger.filename_builder import FilenameBuilder
from gtagger.paperless import Paperless, event_dict
from gtagger.document_processing import DocumentPostProcessor

from nlsearch.store import Store
from nlsearch.config import config as store_config


PROMPT_DOCUMENT = """
Analizza il testo fornito e genera un output in formato JSON.

Assicurati che l'output sia preciso e ben strutturato. Restituisci solo l'oggetto JSON, senza ulteriori testi o spiegazioni.
Note:
- Un "comunicato stampa" è un testo redatto dall'ufficio stampa di VeronaFiere per informare su un fatto oppure su un avvenimento in maniera ufficiale.
- Un "comunicato stampa di terzi" è un testo redatto dall'ufficio stampa di un'azienda o di un ente pubblico per informare su un fatto oppure su un avvenimento in maniera ufficiale di Verona Fiere.
- Un "discorso" è un intervento specifico, tenuto da una singola persona che argomenta a favore di una tesi. E' organizzato per temi, sviluppa argomentazioni e può presentare dati a supporto.
- Una "intervista" è una forma di dialogo strutturato in cui una persona (l'intervistatore) pone delle domande a un'altra persona o a un gruppo di persone (l'intervistato/gli intervistati) con lo scopo di ottenere informazioni, opinioni, testimonianze o approfondimenti su un argomento specifico.
- Un "redazionale"  è un articolo o un testo di carattere giornalistico, che ha lo scopo di presentare un'azienda, un prodotto, un servizio o un evento in modo informativo e promozionale, ma con un tono e uno stile simili a quelli di un contenuto editoriale "neutro". Può essere anche un articolo di giornale inviato a riviste.
- Un "social" è un documento dove ogni paragrafo è un messaggio scritto per essere pubblicato su un social, per esempio Facebook o FB, Instagram o IG, Linkedin.
- Un "invito" è l'offerta fatta ad una persona, ente, azienda di partecipare ad una manifestazione o evento.
- Gli eventi sono classificati come "ARTVERONA, CORPORATE, EOS, FIERACAVALLI, FIERAGRICOLA, JOB&ORIENTA, LETEXPO, MARMOMAC, MCTER EXPO, MINERAL MAGGIO, MINERAL NOVEMBRE, MODEL EXPO, MOTORBIKE, PAVING, PROGETTO FUOCO, SAMOTER, SOL2EXPO, SPORT EXPO, VINITALY, AND THE CITY, VITA IN CAMPAGNA, WINE2WINE"
- Una "scaletta" è un elenco strutturato di attività che si svolgono durante un evento, hanno la struttura di programma o calendario che include orari, partecipanti, e altri dettagli necessari per la sua esecuzione.
- Un "programma" è l'elenco di tutte le attività di un evento, che si svolgono in più giornate o date."""


# PROMPT_DATE = """
# Extract date from the attached row
# Dates appear in various formats and could be only the year
# generate a well formatted JSON file, with this structure
# {"row": text of row, "date": [extracted date  in ISO format]}
# If only the year is found, use only the year, do not invent.
# Return only the JSON object, without any further text or explanation.
# Return to maximum 10 dates.
# """

PROMPT_DATE = """Esamina la stringa allegata: 
 
genera un JSON ben fomattato con le seguenti informazioni disposte per colonna:

    campo "row" con la stringa esaminata

    campo "date"  con la data che  identifichi in formato ISO, usando XX per le parti non rilevate

    campo "persona": ["cognomi che identifichi nella stringa"]

    campo "info" è una stringa ottenuta così: elimina dalla stringa esaminata le informazioni che hai usato per la  data, i cognomi trovati e le seguenti parti:
    ["VINITALY", "FIERACAVALLI","FIERAGRICOLA","cs", "gb" , "gbcs", "riferimenti a lingua usata"] 
    
""" 

config = get_config()

log = logging.getLogger(__name__)

def check_if_already_uploaded(document_path: Path, db_file: Path):
    """Controlla se un file 'risulta' caricato in paperless.
    
    Questo non assicura che il file sia ancora presente nell'archivio, ma 
    solo che l'operazione di caricamento è stata effettuata con successo"""
    uploaded = False
    if not db_file.exists():
        return uploaded
    with open(db_file, 'r') as f:
        for line in f:
            path = line.split(';')
            print(path)
            if document_path.as_posix() == path[0]:
               uploaded = True
               break

    return uploaded


def do_markitdown(filepath: str):
    md = MarkItDown()
    log.info(f"Conversione del file {filepath}")
    try:
        result = md.convert(filepath)
    except FileConversionException as e:
        log.error(f"La conversione del file {filepath} ha prodotto l'errore: {e}.")
        return None
    except UnsupportedFormatException as e:
        log.error(f"Formato del file {filepath} non supportato.")
        return None
    except:
        log.error(
            f"La conversione del file {filepath} ha prodotto un errore non gestito"
        )
        return None
    return result


def check_if_scanned_pdf(file: str, target_file: str):
    """
    Converte un file pdf da scansione in testo.
    """
    try:
        log.info(f"OCR del file {file}")
        with tempfile.NamedTemporaryFile(mode="w+", delete=False) as tmp_file:
            ocrmypdf.ocr(
                input_file=file,
                output_file=tmp_file.name,
                language="ita",
                force_ocr=True,
                max_image_mpixels=600,
            )
            tmp_path = tmp_file.name
            log.debug(f"Creato file temporaneo OCR {tmp_path}")
            result = do_markitdown(tmp_path)
            if result:
                with open(target_file, "w") as f:
                    f.write(result.text_content)
                    log.debug(f"File markdown per {file} creato.")
        if tmp_path:
            os.remove(tmp_path)
    except ocrmypdf.exceptions.TaggedPDFError as e:
        log.error(f"File {file} marcato come tagged")
        # Questa eccezione è sollevata se il PDF è marcato come TAGGED
        # --force-ocr per forzare
        raise

def semantic_rebuild(file_list):
    store = Store(
        path=store_config.db,
        collection=store_config.collection,
        model_name=store_config.semantic_model,
        semantic_chunker=store_config.semantic_chunker,
        chunk_size=store_config.chunk_size,
        chunk_overlap=store_config.chunk_overlap,
        
    )
    tagger = Tagger()
    result = tagger.traverse(file_list)
    documents = [
        file for file in result if os.path.splitext(file)[1].lower() in config.extension
    ]

    for document in documents:
        basename = os.path.splitext(document)[0] 
        already_uploaded_path = basename + ".ok"
        if not os.path.exists(already_uploaded_path) :
            log.warning(f"Documento {document} non presente su paperless")
            continue
        log.info(f"semantic rebuild {basename}")
        with open(already_uploaded_path) as f:
            parent_id = f.read().strip()
        if parent_id:
            log.info(f"update_vector_store parent_id={parent_id} document={document}")
            payload_path = basename + ".md"
            if os.path.exists(payload_path):
                store.add_file(payload_path, parent_id=parent_id)

def upload_document(file_list):
    # 1) Recupero lista files da albero
    # 2) Recupero id campi da caricare
    # 3) controllo se non devo archiviare
    # 4) carico Documento da json
    # 5) filtering caricamento (contabilità, carte identità...)
    # 6) costruzione Payload
    # 7) estrazione tags (per ora solo anno)
    # 8) post documento
    # 9) check:
    #    - se ok custom fields
    #    - write OK
    #10) Vector Store
    from gtagger.file_explorer import FileExplorer
    from gtagger.model import data_load

    store = Store(
        path=store_config.db,
        collection=store_config.collection,
    )

    #### 1
    file_explorer = FileExplorer(file_list)
    file_explorer.traverse()

    #### 2
    correspondents = Paperless.get_correspondents()
    document_types = Paperless.get_document_types()
    
    for file in file_explorer.files.get(".docx", []):
        json_source_path = file.with_suffix(".json")
        
        #### 3
        already_uploaded_path = file.with_suffix(".ok")

        if check_if_already_uploaded(file, already_uploaded_path):
            continue
        
        #### 4
        document = data_load(json_source_path)

        #### 5
        # Se è un documento contabile NON VA CARICATO
        # non rifattorizzo questo aspetto finchè il filtering
        # è solo questo
        if document.tipo_documento == "cont":
            # TODO INFO o WARNING?????
            log.info(f"Documento {document} non caricato perché di tipo contabile")
            continue

        #### 6

        payload = {"title": file.stem, "created": document.data_evento.strftime("%Y-%m-%d")}
        
        # Estraggo evento
        radice = Path(config.files[0]).parent.resolve()
        directory_evento = file.relative_to(radice).parts[0]
        evento = event_dict.get(directory_evento, "Sconosciuto")

        # Correspondent
        for correspondent in correspondents:
            if correspondent.name == directory_evento:
                payload["correspondent"] = correspondent.id
                break
        
        # Document Typedocument_dict
        # # TODO se non dovesse esserci il tipo di documento non faccio l'upload e lo segnalo

        dc = document_dict.get(document.tipo_documento)
        for document_type in document_types:
            if document_type.name == dc:
                payload["document_type"] = document_type.id
                break
        # Anno come tag
        tags = Paperless.get_tags()

        anno_evento = (
            str(document.data_evento.year)
            if document.data_evento
            else None
        )
        if anno_evento:
            tag_id = None
            for tag_anno in tags:
                if tag_anno == anno_evento:
                    tag_id = tags[tag_anno]
                    break
            if not tag_id:
                tag_id = Paperless.post_tag(anno_evento)
            if tag_id:
                payload["tags"] = [tag_id]
        log.debug(payload)
        
        paper = Paperless()
        task_id = paper.post_document(file, payload)
        result = paper.taskid_check(task_id)
        parent_id = None

        if result:
            parent_id = result.get("related_document")
            if result.get("status") == "SUCCESS":

                
                
                log.warning(f"Documento {document} caricato")
                # mando i custom fields usando il document id
                paperless_custom_fields = paper.get_custom_fields()
                cf = CustomField(document, evento, paperless_custom_fields, file.stem)

                # FIX: togliere il codice dal costruttore di CustomField 
                # probabilmente questo campo verrà invertito con Title
                filename_builder = FilenameBuilder()
                name = filename_builder.build_filename(document, evento, original_filename=file.as_posix())
                cf.filename = name

                cf_obj = cf.custom_field_object()
                response = paper.update_custom_fields(
                    result["related_document"], cf_obj
                )
                with open(already_uploaded_path, 'a') as f:
                    line = "\n" + ";".join([file.as_posix(),parent_id])
                    f.write(line)
            else:
                #parent_id = result.get("related_document")
                
                resp = result.get("result")
                if resp.lower().find("it is a duplicate") != -1:
                    log.warning(f"Trovato duplicato non marcato come caricato")
                    # TODO GEstire il file ok in questa situazione
                    # with open(already_uploaded_path, 'w') as f:
                    #     f.write(result.get("related_document","-1"))
                log.error(f"Fallito caricamento, response={resp}")

        if parent_id:
            log.info(f"update_vector_store parent_id={parent_id} document={document}")

            #update_vector_store(parent_id, document)
            payload_path = file.with_suffix(".md") 
            if os.path.exists(payload_path):
                store.add_file(payload_path, parent_id=parent_id)

# rifattorizzato. Resta temporaneamente
def old_upload_document(file_list):
    tagger = Tagger()
    result = tagger.traverse(file_list)
    documents = [
        file for file in result if os.path.splitext(file)[1].lower() in config.extension
    ]
    correspondents = Paperless.get_correspondents()
    document_types = Paperless.get_document_types()

    store = Store(
        path=store_config.db,
        collection=store_config.collection,
        model_name=store_config.semantic_model,
        semantic_chunker=store_config.semantic_chunker,
        chunk_size=store_config.chunk_size,
        chunk_overlap=store_config.chunk_overlap,
    )

    # E' un test
    # Prendo i pdf nella directory. Se c'e' un json corrispondente procedo
    # for document in list(source_path.glob("*.pdf")):
    for document in documents:
        json_source_path = os.path.splitext(document)[0] + ".json"
        already_uploaded_path = os.path.splitext(document)[0] + ".ok"
        if not os.path.exists(json_source_path) :
            log.warning(f"Documento {document} non classificato da AI")
            continue
        if os.path.exists(already_uploaded_path):
            log.warning(f"Documento {document} già archiviato")
            continue


        paper = Paperless()
        data = paper.import_from_json(json_source_path)

        # Se è un documento contabile NON VA CARICATO
        if data.get("tipo_documento") and data.get("tipo_documento") == "cont":
            # TODO INFO o WARNING?????
            log.info(f"Documento {document} non caricato perché di tipo contabile")
            continue

        # Se la data è invalida, uso il 1 gennaio 1900 per distingurla
        data_evento = parse_iso_date(data["data_evento"])
        if data_evento is not None:
            data["data_evento"] = data_evento
        else:
            data["data_evento"] = datetime(1900, 1, 1)

        #
        documento = Document(
            persone_citate=data["persone_citate"] if data.get("persone_citate") else [],
            relatori=data["relatori"] if data.get("relatori") else [],
            data_evento=data["data_evento"],
            macroargomenti_trattati=data["macroargomenti_trattati"]
            if data.get("macroargomenti_trattati")
            else [],
            tipo_documento=data["tipo_documento"],
        )

        # Senza estensione
        title = os.path.basename(document)
        title = os.path.splitext(title)[0]

        ### Lingua documento
        md_source_path = os.path.splitext(document)[0] + ".md"
        tika = TikaConverter()
        language = tika.get_language(Path(md_source_path))
        log.debug(f"Linguaggio documento: {language}")
        if data["tipo_documento"] == "cs" and language == "en":
            data["tipo_documento"] == "pr"
        if data["tipo_documento"] == "dis" and language == "en":
            data["tipo_documento"] == "sp"

        # Regola salvataggio per non riprocessare tutto con l'ia, TEMPORANEA
        if data["tipo_documento"] == "pr" and language == "it":
            data["tipo_documento"] == "cs"
        if data["tipo_documento"] == "sp" and language == "it":
            data["tipo_documento"] == "dis"


        payload = {"title": title, "created": data["data_evento"].strftime("%Y-%m-%d")}

        # Estraggo evento
        radice = Path(config.files[0]).parent.resolve()
        directory_evento = Path(document).relative_to(radice).parts[0]
        evento = event_dict.get(directory_evento, "Sconosciuto")

        # Correspondent
        for correspondent in correspondents:
            if correspondent.name == directory_evento:
                payload["correspondent"] = correspondent.id
                break

        # Document Typedocument_dict
        # # TODO se non dovesse esserci il tipo di documento non faccio l'upload e lo segnalo

        dc = document_dict.get(documento.tipo_documento)
        for document_type in document_types:
            if document_type.name == dc:
                payload["document_type"] = document_type.id
                break

        # Anno come tag
        tags = Paperless.get_tags()

        anno_evento = (
            str(documento.data_evento.year)
            if documento.data_evento
            else None
        )
        if anno_evento:
            tag_id = None
            for tag_anno in tags:
                if tag_anno == anno_evento:
                    tag_id = tags[tag_anno]
                    break
            if not tag_id:
                tag_id = Paperless.post_tag(anno_evento)
            if tag_id:
                payload["tags"] = [tag_id]
        log.debug(payload)

        task_id = paper.post_document(document, payload)
        result = paper.taskid_check(task_id)
        parent_id = None

        if result:
            parent_id = result.get("related_document")
            if result.get("status") == "SUCCESS":

                filename_builder = FilenameBuilder()
                filename_builder.build_filename(documento, evento, original_filename=document)
                
                
                log.warning(f"Documento {documento} caricato")
                # mando i custom fields usando il document id
                paperless_custom_fields = paper.get_custom_fields()
                cf = CustomField(documento, evento, paperless_custom_fields, title)
                cf_obj = cf.custom_field_object()
                response = paper.update_custom_fields(
                    result["related_document"], cf_obj
                )
                with open(already_uploaded_path, 'w') as f:
                    f.write(parent_id)
            else:
                #parent_id = result.get("related_document")
                resp = result.get("result")
                if resp.lower().find("it is a duplicate") != -1:
                    log.warning(f"Trovato duplicato non marcato come caricato")
                    with open(already_uploaded_path, 'w') as f:
                        f.write(result.get("related_document","-1"))
                log.error(f"Fallito caricamento, response={resp}")
        if parent_id:
            log.info(f"update_vector_store parent_id={parent_id} document={document}")
            #update_vector_store(parent_id, document)
            payload_path = os.path.splitext(document)[0] + ".md"
            # if os.path.exists(payload_path):
            #     store.add_file(payload_path, parent_id=parent_id)


def patch_document(file_list):
    """Ricarica tutti i dati di un documento"""
    from gtagger.file_explorer import FileExplorer
    from gtagger.filename_builder import FilenameBuilder
    from gtagger.model import Document, data_load, event_dict

    documents = FileExplorer(file_list)
    documents.traverse()
    filename = FilenameBuilder()
    
    # Scorri i files ".ok"
    # prendi id 
    # Carica documento
    # 

    # file = Path("/home/venom/Progetti/paperless/galileo/experiments/samples/VeronaFiereTest/TEST/Intervista/Federico Bricolo - Intervista_L'Espresso.ok")
    # id = 182
    # json_source_path = file.with_suffix(".json")

    paperless = Paperless()
    for group in set(config.extension):
        for file in documents.files.get(".ok", []):
            json_source_path = file.with_suffix(".json")
    
            with open(file, 'r') as f:
                for line in f:
                    if line.strip() == "":
                        continue
                    path, id = line.split(';')
                    new_document = data_load(json_source_path)

                    # Costruisco il nuovo payload
                    data_evento = new_document.data_evento if isinstance(new_document.data_evento, str) \
                                                        else new_document.data_evento.strftime("%Y-%m-%d")
                    payload = {"title": json_source_path.stem, "created": data_evento}

                    # Tirare fuori dai cicli
                    correspondents = Paperless.get_correspondents()
                    document_types = Paperless.get_document_types()

                    # Estraggo evento
                    file.relative_to(documents.root).parts[0]
                    radice = Path(config.files[0]).parent.resolve()
                    directory_evento = "TEST" #file.relative_to(radice).parts[0]
                    evento = event_dict.get(directory_evento, "Sconosciuto")

                    # Correspondent
                    for correspondent in correspondents:
                        if correspondent.name == directory_evento:
                            payload["correspondent"] = correspondent.id
                            break

                    # Document_type
                    dc = document_dict.get(new_document.tipo_documento)
                    for document_type in document_types:
                        if document_type.name == dc:
                            payload["document_type"] = document_type.id
                            break
                
                    #tags
                    tags = Paperless.get_tags()

                    anno_evento = (
                        str(new_document.data_evento.year)
                        if new_document.data_evento
                        else None
                    )
                    if anno_evento:
                        tag_id = None
                        for tag_anno in tags:
                            if tag_anno == anno_evento:
                                tag_id = tags[tag_anno]
                                break
                        if not tag_id:
                            tag_id = Paperless.post_tag(anno_evento)
                        if tag_id:
                            payload["tags"] = [tag_id]
                    
                    response = paperless.update_document(id, payload)

                    if response:
                        ### Custom fields new
                        filename_builder = FilenameBuilder()
                        filename = filename_builder.build_filename(new_document, evento, original_filename=file.as_posix())
                       
                        log.warning(f"Documento {new_document} caricato")
                        # mando i custom fields usando il document id
                        paperless_custom_fields = paperless.get_custom_fields()
                        cf = CustomField(new_document, evento, paperless_custom_fields, file.stem)
                        cf.filename = filename
                        cf_obj = cf.custom_field_object()
                        response = paperless.update_custom_fields(
                            id, cf_obj
                        )

# Usato per fare test, da togliere
def patch_document_(file_list):
    # Bozza di patch Documenti
    from gtagger.file_explorer import FileExplorer
    from gtagger.filename_builder import FilenameBuilder
    from gtagger.model import Document, data_load, event_dict

    documents = FileExplorer(file_list)
    documents.traverse()
    filename = FilenameBuilder()
    
    for group in set(config.extension):
        # log.info(group)
        # log.info(documents.root)
        # log.info(documents.files.keys())
        for file in documents.files.get(group, []):
            
            json_source_path = file.with_suffix(".json")
            if not json_source_path.exists():
                continue
            document = data_load(json_source_path)
    
            relative_path = file.relative_to(documents.root)
            persona = document._filename_info.get("persona","")
            info = document._filename_info.get("info","")
            tipo_evento = event_dict.get(relative_path.parts[0],"xxx")
            new_filename = filename.build_filename(document, tipo_evento, file.stem)
            print(file.parts[1], file.name, new_filename + persona + info)
    log.info(f"{documents.root}")
    log.info("--- end ---")


def do_stats(file_list):
    #from pympler import asizeof

    stats = StatsCollector(config.files[0])

    stats.report()
    # total_size = asizeof.asizeof(stats)
    # log.debug (f"Memoria usata da  istanza statistiche: {total_size / (1024 * 1024):.2f} MB")
    stats.save_report_as_csv()
    stats.save_report_as_json()
 

# Non sta bene mel main
def convert(source_path: str, extensions: list):
    """Converte files in markdown.

    Esplora ricorsivamente un percorso e converte tutti quelli che hanno estensione ammessa.

    Args:
        source_path (str): la stringa contentente il path del file
        extensione  (list): la lista delle estensioni consentite

    Returns:
        None

    """
    tagger = Tagger()
    files = tagger.traverse(source_path, result=False)

    # TODO Finalizzare il test tika/markitdown

    # path = Path(filepath)

    converters = [Convert("tika"), Convert("markitdown")]

    for file in files:
        log.debug(f"processing {file}")
        base, extension = os.path.splitext(file)

        if extension.lower() in extensions:
            target_file = base + ".md"
            if os.path.exists(target_file):
                log.debug(f"{target_file} present, skip")
                continue

            for converter in converters:
                try:
                    result = converter.extract_text(Path(file))
                    break
                except ConversionError as e:
                    log.error(str(e))
                    continue
            else:
                raise ConversionError(f"Impossibile convertire il file: {file}")

            log.debug(len(result))
            if result:
                with open(target_file, "w") as f:
                    f.write(result)
                    log.info(f"markdown {target_file} created")
            elif result is not None and extension == ".pdf":
                check_if_scanned_pdf(file, target_file)


# TODO razionalizzare i basename etc...
def ai(path):
    tagger = Tagger()
    llm = LLMInvoker()
    all_files = tagger.traverse(path)

    MD_EXT = ".md"

    # Deprecato: https://gitea.csgalileo.org/galileo/paperless/issues/5
    # Filtri nomefile -> tipo documento
    # filters = [(".*gbcs[0-9]*[ _-]+", "pr"), (".*cs[0-9]*[ _-]+","cs")]

    for source_file in all_files:
        # Processa solo i documenti con estensione opportuna
        if not os.path.splitext(source_file)[1] in config.extension:
            continue

        target_file = os.path.splitext(source_file)[0] + MD_EXT
        if not os.path.exists(target_file):
            continue

        with open(target_file, "r") as f:
            testo_da_elaborare = f.read()

        log.debug(f"checking {target_file}")

        tag_file = os.path.join(target_file, os.path.splitext(target_file)[0] + ".json")
        if os.path.exists(tag_file):
            log.debug(f"JSON {tag_file} present: skip")
            continue

        document_post_processor = DocumentPostProcessor()
        # Conta token. Se sotto un certo limite, non elaborare
        tokens = testo_da_elaborare.split()
        if len(tokens) < 100:
            tagged_result = Document(persone_citate=[], relatori=[], data_evento="1970", 
                                    tipo_documento="img",macroargomenti_trattati=[], 
                                    lingua_utilizzata="")

            data_estratta = ""
        else:
            # TODO gestire esaurimento risorse
            # google.api_core.exceptions.ResourceExhausted
            
            tagged_result, usage_data = llm.analyze(
                PROMPT_DOCUMENT, testo_da_elaborare, template=Document
            )
            assert tagged_result is not None
            assert usage_data is not None

            document_post_processor.apply_language_rules(tagged_result, target_file)

            # Controllo data mandando  all'ai il nome del file
            # [{"row": nomefile, "date: []"}]
            # Un po' debole.
            basename = os.path.basename(source_file)
            log.debug(f"Ricerca data in {basename}")
            response, _usage_filename = llm.analyze(
                PROMPT_DATE, os.path.splitext(basename)[0]
            )
            
            assert response is not None
            assert _usage_filename is not None
            tagged_result._usage_filename = _usage_filename
            tagged_result._usage_data = usage_data


            if isinstance(response, list):
                response = response[0]

            log.debug(f"risultato ricerca data {response}")
            date_field = response.get("date", list())
            data_estratta = date_field[0] if len(date_field) > 0 else ""
            tagged_result._filename_info = response
        
        # if DateHelper.is_acceptable_data(data_estratta):
        #     tagged_result.data_evento = data_estratta
        # ###### sotto test
        # elif parse_iso_date(tagged_result.data_evento) is None:
        #     tagged_result.data_evento = DateHelper.get_last_change(source_file)

        
        document_post_processor.apply_date_rules(tagged_result, source_file, tagged_result._filename_info)

        #### Tipo documento di default se l'IA non crea il campo ####
        if not tagged_result.tipo_documento:
            tagged_result.tipo_documento = "def"


        # Deprecato https://gitea.csgalileo.org/galileo/paperless/issues/5
        # #### Tipo documento da nome file
        # gb uk 
        # # "cs[0-9]*[ _-]+"
        # basename = os.path.basename(source_file)
        # basename = os.path.splitext(basename)[0]

        # for filter, value in filters:
        #     if re.match(filter,basename.lower()):
        #         log.debug(f"Ricavato tipo documento {value} da nome file {source_file}")
        #         tagged_result["tipo_documento"] = value
        #         break

        # Scrittura risultati
        with open(tag_file, "w") as f:
            json.dump(tagged_result.full_serialize(), f)
            log.info(f"{tag_file} created")
        # FIXME Provvisorio durante le prove per superare i limiti di token per minuto
        import time

        time.sleep(3)


def setup_paperless():
    paper = Paperless()
    paper.add_correspondents()
    paper.add_custom_fields()
    paper.add_document_types()


def main():
    if config.task != "setup" and not len(config.files):
        log.error(f"E' richiesta una directory di documenti.")
        sys.exit(-1)


    match config.task:
        case "convert":
            convert(config.files[0], config.extension)
        case "ai":
            ai(config.files[0])
        case "report":
            do_stats(config.files[0])
        case "upload":
            upload_document(config.files[0])
        case "semantic-rebuild":
            semantic_rebuild(config.files[0])
        case "patch":
            patch_document(config.files[0])
        case "setup":
            setup_paperless()
