from typing import List, Any, Optional, Dict, Literal, TypeAlias, Union, get_args
from typing_extensions import Annotated, TypedDict
from pydantic import BaseModel, Field
from datetime import datetime, date
from dataclasses import dataclass
import json
import logging

from gtagger.config import get_config
config = get_config()
assert config is not None, "La configurazione non è inizializzata correttamente"

log = logging.getLogger(__name__)

MAX_LENGTH = 128

# L'AI contrassegna il tipo di documento con i valori che ho messo in Key
# I values() sono invece i valori come li vede paperless (custom field: Tipo di Documento)
# Le keys() sono usate anche per costruire il custom field: Nome File
document_dict = {
    "cs": "Comunicato Stampa",
    "sp": "Speech",
    "cst": "Cartella Stampa",
    "red": "Redazionali",
    "int": "Intervista",
    "prg": "Programma",
    "scal": "Scaletta",
    "rep": "Report",
    "soc": "Social",
    "std": "Save The Date",
    "inv": "Inviti Stampa",
    "cs3": "Comunicati Stampa Terzi",
    "pk": "Press Kit",
    "pr": "Press Release",
    "img": "Immagine",
    "dis": "Discorso",
    "def": "Generale"
}


# Ocio: Python 3.11+ 
# Serve alla classe Document
# Aggiungo dei tipi con non sono in Paperless, e che mi servono per filtrare i files
TipoDocumento : TypeAlias = Literal[
    "cs", "sp", "cst", "red", "int", "prg", "scal", "rep", "soc",
    "soc", "std", "inv", "cs3", "pk", "pr", "img", "dis", "def", "riepilogo", "cont"
]


# Verifica a runtime che le chiavi del dizionario corrispondano al Literal 
# Purtoppo per usare pylance bisogna scendere a compromessi.
assert set(get_args(TipoDocumento)) - set(
    document_dict.keys()) == {"riepilogo", "cont"}, "Le chiavi non corrispondono al tipo!"

# Mappa del tipo evento
# keys() sono la prima sottodirectory dell'archivio (i il corrispondent in Paperless)
# values() come lo distinguiamo in paperless (custom field: Tipo Evento)
event_dict = {
    "ARTVERONA": "art",
    "CORPORATE": "gen",
    "EOS": "eos",
    "FIERACAVALLI": "fic",
    "FIERAGRICOLA": "agr",
    "JOB&ORIENTA": "job",
    "LETEXPO": "tra",
    "MARMOMAC": "mdm",
    "MCTER EXPO": "eio",
    "MINERAL MAGGIO": "min",
    "MINERAL NOVEMBRE": "geo",
    "MODEL EXPO": "mod",
    "MOTORBIKE": "mbe",
    "PAVING": "pav",
    "PROGETTO FUOCO": "fuo",
    "SAMOTER": "sam",
    "SOL2EXPO": "sag",
    "SPORT EXPO": "spo",
    "VINITALY": "vir",
    "AND THE CITY": "vac",
    "VITA IN CAMPAGNA": "vca",
    "WINE2WINE": "w2w",
    "TEST": "tst"
}

# Con pydantic posso permettermi l'implementazione di eventuali metodi e la validazione dei dati
class Document(BaseModel):    
    persone_citate: List[str] = Field(
        default_factory=list,
        description = "Elenco di nomi di persone citate (es. artisti, curatori, figure rilevanti)."
    )

    data_evento: Union[date, str] = Field(
        "XXXX-XX-XX",
        description = "Data di riferimento dell'evento (formato YYYY-MM-DD)."
    )

    relatori: List[str] =  Field(
        default_factory=list,
        description = "Elenco di nomi dei relatori, chi commenta, chi pronuncia un discorso, chi viene intervistato se presenti nel testo."
    )

    macroargomenti_trattati: List[str] = Field(
        default_factory=list,
        description = "Elenco dei temi principali trattati (es. sostenibilità, arte, ambiente)."
    )

    # Impostato con Tika
    lingua_utilizzata: str = Field(
        "it",
        description = "la lingua principale del testo in codice standard standard ISO 639-1"
    )

    tipo_documento: TipoDocumento =  Field(
        ...,
        description = """Classifica il documento scegliendo tra i seguenti codici:
        - cs: Comunicato stampa emesso dalla redazione di Verona Fiere in cui l'enfasi è sull'ente fieristico e sull'evento principale organizzato.La prospettiva è quella dell'ente organizzatore.
        - cst: Cartella di comunicati (riepilogo di comunicati stampa).
        - red: Redazionali (articoli di giornale inviati a riviste).
        - sp: discorso in lingua inglese.
        - dis: discorso in lingua italiana
        - scal: Scaletta.
        - rep: Report (resoconto di riunione).
        - soc: Social (testi per piattaforme social).
        - std: Inviti.
        - int: intervista.
        - cs3: comunicato stampa emesso da un terzo in cui l'enfasi è sul ruolo e le attività del terzo all'interno dell'evento.La prospettiva è quella dell'entità esterna che partecipa o collabora. I riferimenti o i contatti, spesso sono riferiti al terzo.
        - cont: preventivo, fattura, ricevuta fiscale, documento contabile, offerta economica"""
    )

    _usage_filename: dict = {}
    _usage_data: dict = {}
    _filename_info: dict = {}

    def full_serialize(self) -> dict:
        data_to_save = self.model_dump(exclude=None)
        # La data non è serializzabile, perciò devo convertirla in stringa
        if isinstance(data_to_save["data_evento"], date):
            data_to_save["data_evento"] = data_to_save["data_evento"].strftime("%Y-%m-%d")
        data_to_save.update({
            "_usage_filename": self._usage_filename,
            "_usage_data": self._usage_data,
            "_filename_info": self._filename_info
        })
        return data_to_save
    
def data_load(filename) -> Document:
    # Apre un file JSON (risultante da AI) per restituire un oggetto Document
    # Se il documento nnon passa la validazione solleva una eccezione

    # raise: pydantic_core._pydantic_core.ValidationError
    try:
        with open(filename, 'r', encoding='utf-8') as file:
            data = json.load(file)
    except (FileNotFoundError, json.JSONDecodeError) as e:
        logging.error(f"Errore nel caricamento del file '{filename}': {e}")
        raise
    
    document = Document.model_validate(data)
        #document = document.model_validate(data) # Popola l'istanza di Document con i dati caricati.
    
    # Vogliamo una data ISO YYYY-MM-DD
    # FIX Adesso è None, ma dovrà errere la data di modifica (operazione delegata non correttamente a main
    document.data_evento = parse_iso_date(document.data_evento)
    if data.get("_usage_data") is not None:
        document._usage_data = data.get("_usage_data")
    if data.get("_usage_filename") is not None:
        document._usage_filename = data.get("_usage_filename")
    if data.get("_filename_info") is not None:
        document._filename_info = data.get("_filename_info")
    
    return document


def parse_iso_date(date_str: str) -> datetime | None:
    """Converte una stringa ISO in datetime o restituisce None se fallisce."""
    try:
        return datetime.fromisoformat(date_str)
    except ValueError:
        logging.warning(f"Formato data non valido: {date_str}")
        return None
    except TypeError:
        # date_str == None significa che l'operazione di analisi non ha fornito nessuna data
        logging.warning(f"Data mancante")
        return None


# I custom fiels hanno delle limitazioni, in relazione al tipo.
# In particolare i tipi String ammettono al più 128 caratteri
# Qui assicuriamo il rispetto del limite.
def format_field(value: Any, max_length: int = MAX_LENGTH) -> Any:
    if isinstance(value, (list, set)):
        # Controllo il limite di lunghezza. Ovviamente non si tronca
        # la stringa risultante, ma la si compone con elementi completi entro 
        # la MAX_LENGTH
        length = 0
        temp_list = []
        for i in value:
            length = length + len(i) + (1 if temp_list else 0)
            if length > max_length:
                break
            temp_list.append(i)
        field = ",".join(temp_list)
    elif  isinstance(value, int):
        field = value
    else:
        field = str(value)[:max_length]
    return field




# CUSTOM FIELDS MAPPING
# Evitiamo sorprese in caso di modifiche in Paperless
# chiave: nome attributo classe Document (es self.anno)
# valore: nome Custom Field in Paperless (es Anno), QUI scritto in MINUSCOLO
property_to_name_map = {
            "persone_citate": "persone citate",
            "argomenti": "argomenti",
            "anno": "anno",
            "tipo_evento": "tipo evento",
            "persone_evidenziate": "persone evidenziate",
            "relatori": "relatori",
            "filename" : "nome file"
        }


# Correspondent Come lo ricevo in JSON da Paperless
@dataclass
class Correspondent:
    id: int
    slug: str
    name: str
    match: str
    matching_algorithm: int
    is_insensitive: bool
    document_count: int
    owner: int
    user_can_change: bool 
    

# Document Type Come lo ricevo in JSON da Paperless
@dataclass
class DocumentType:
      id: int
      slug: str
      name: str
      match: str
      matching_algorithm: int
      is_insensitive: bool
      document_count: int
      owner: int
      user_can_change: bool


# Custom Field Come lo ricevo in JSON da Paperless
@dataclass
class JsonItem:
    id: int
    name: str
    data_type: str
    extra_data: dict
    document_count: int 


# Custom Field come lo mando a Paperless
@dataclass
class FinalItem:
    id: int
    value: str


# Sono i "Campi personalizzati" di paperless. La maggiorparte di essi la si ritrova nel .json
# Uso la classe Document per caricare i dati dal file su disco e poi li estraggo
class CustomField:
    def __init__(self, documento: Document, tipo_evento: str, paperless_cf: Optional[List[Dict]]=None, filename = ""): 
        if paperless_cf is None:
            paperless_cf = []
        self.persone_citate = documento.persone_citate
        self.argomenti = documento.macroargomenti_trattati
        self.anno = documento.data_evento.year
        self.tipo_evento = tipo_evento
        self.relatori = documento.relatori
        self.paperless_custom_fields = [JsonItem(**item) for item in paperless_cf]
        
        # Persone citate che hanno particolare evidenza
        self.persone_evidenziate = set(config.person) & {x.title() for x in self.persone_citate}
        self.relatori = documento.relatori if documento.tipo_documento in ("sp", "dis", "red", "int") else []
        # if not documento.tipo_documento:
        #     raise ValueError("Il campo 'tipo_documento' è obbligatorio")
        # if not documento.data_evento:
        #     raise ValueError("Il campo 'data_evento' è obbligatorio per costruire il nome del file")

        # # questa dovrebbe errere migliore
        # data_evento = documento.data_evento
        # # Paranoia. Se succede è un problema.
        # assert data_evento is not None
        # data_evento = data_evento.strftime("%Y_%m_%d")
        
        # filename = filename.lower()
        # ### Costruzione Nome file 
        # ### TODO refactor to method
        # filename_tmp = [
        #     documento.tipo_documento, self.tipo_evento
        # ]  

        # # FIX la persona nel nome file solo per speech e discorsi 
        # # Barbara scrive anche appunti e saluti, ma non abbiamo tali tipi di documento
        # # La persona è nel filename 
        # persone_evidenziate = []
        # if documento.tipo_documento in ("sp", "dis"):
        #     # Persone evidenziate nel nome file...
        #     cognomi = [p.split()[-1].lower() for p in config.person]
        
        #     for cognome in cognomi:
        #         if filename.find(cognome) == -1:
        #             continue
        #         persone_evidenziate.append(cognome)
        #     persone_evidenziate = "_".join(persone_evidenziate)

        
        # filename_tmp.append(data_evento)
        # if len(persone_evidenziate) > 0: 
        #     filename_tmp.append(persone_evidenziate)
        # self.filename = "-".join(filename_tmp)
        ### fine costruzione nome file
            
    # Faccio il mapping tra la proprietà (es self.persone_citate) e l'id del 
    # corrisondente custom field in paperless per la HTTP patch
    #Es extracted_ids = {'argomenti': 2, 'persone_citate': 3}
    def extract_ids_from_json(self, json_data: List[JsonItem]) -> dict:
        # Dizionario per memorizzare gli id estratti
        extracted_ids = {}

        # Scansiona il JSON per trovare gli id corrispondenti
        #property_name=="tipo_evento"  Proprietà Classe CustomField
        #json_name=="tipo evento" 
        #item.name== "Persone Citate" Nome Proprietà in paperless associata ad un id che devo ricavare
        for item in json_data:
            for property_name, json_name in property_to_name_map.items():
                if item.name.lower() == json_name:
                    extracted_ids[property_name] = item.id

        return extracted_ids   
    

    def custom_field_to_dict(self):
        # Ritorna lo stato della classe come dizionario
        return {attr: value for attr, value in self.__dict__.items() if attr in property_to_name_map}


    # Creo l'oggetto che usetò nella HTTP patch per caricare il custom field
    # FinalItem(id=2, value=['vini', 'fiere', 'export'])]
    def custom_field_object(self) -> List[FinalItem]:
        # Estrai gli id dal JSON
        extracted_ids = self.extract_ids_from_json(self.paperless_custom_fields)

        # Costruisci l'oggetto finale utilizzando i dataclass
        # 1) aggiungo a paperless solo quello che c'e'
        # 2) basta aggiunge o togliere un campo a property_to_name_map e alla classe senza toccare la funzione
        custom_field_object = []
        for attr, json_key in property_to_name_map.items():
            field_id = extracted_ids.get(attr)
            field_value = getattr(self, attr, None)
            if field_id is not None and field_value:
                formatted_value = format_field(field_value)
                custom_field_object.append(FinalItem(id=field_id, value=formatted_value))
                logging.debug(f"Aggiunto Field id: {field_id} - Field value: {field_value}")
            else:
                logging.warning(f"Campo personalizzato '{attr}' (ID: {field_id}, Valore: {field_value}) non aggiunto")

        return custom_field_object
    

    def __str__(self):
        return f"Persone citate: {self.persone_citate}\n" + \
               f"Argomenti: {self.argomenti}\n" + \
               f"Anno evento: {self.anno}\n" + \
               f"Tipo Evento: {self.tipo_evento}\n" + \
               f"Persone Evidenziate: {self.persone_evidenziate}\n" + \
               f"Relatori: {self.relatori}\n\n"
    
