#   "_usage_filename": {
#     "input_tokens": 105,
#     "output_tokens": 51,
#     "total_tokens": 156,
#     "input_token_details": {
#       "cache_read": 0
#     }
#   },
#   "_usage_data": {
#     "input_tokens": 1873,
#     "output_tokens": 50,
#     "total_tokens": 1923,
#     "input_token_details": {
#       "cache_read": 0
#     }
#   }



import logging
import json
from collections import defaultdict
from pathlib import Path
from typing import Dict, List
from datetime import datetime
import csv

from gtagger.file_explorer import FileExplorer
from gtagger.config import get_config
from gtagger.tagger import Tagger
from gtagger.model import CustomField, data_load
from gtagger.paperless import  event_dict

config = get_config()
log = logging.getLogger(__name__)

class StatsCollector:
    def __init__(self, filespath: str):
        self.estimated_token_consumption : int = 0
        self.real_token_consumption : Dict[str, Dict[str, int]] = dict()
        self.source_files_count : int = 0
        self.analized_files : int = 0
        self.converted_files : int = 0
        self.processed_files : int = 0
        self.uploaded_files : int = 0
        self.dir_content: Dict = dict()
        self.files : List = []
        self.fields = ("_usage_data", "_usage_filename")
        self.keys = ("input_tokens", "output_tokens", "total_tokens")
        try:
            self.explorer = FileExplorer(filespath)
            self.explorer.traverse()
        except Exception as e:
            raise RuntimeError(f"Errore nell'esplorazione dei file: {e}")

    def token_consumption(self):
        for field in self.fields:
            self.real_token_consumption.setdefault(field, {
                "input_tokens": 0,
                "output_tokens": 0,
                "total_tokens": 0,
            })

        for file in self.explorer.files.get(".json", []):
            try:
                with open(file, 'r') as f:
                    data = json.load(f)
                    for field in self.fields:
                        usage = data.get(field)
                        if isinstance(usage, dict):
                            for key in self.keys:
                                if key in usage:
                                    self.real_token_consumption[field][key] += usage[key]
            except json.JSONDecodeError as e:
                raise ValueError(f"Errore {e} nel parsing del JSON: {file}")


    def token_estimation(self):
        tokens = 0

        for file in self.explorer.files.get(".md"):
            with open(file, "r") as f:
                testo_da_elaborare = f.read()
                tokens += len(testo_da_elaborare.split())
        
        self.token_estimation = tokens / 4


    def document_status(self):
        """Effettua l'attraversamento di un path.
        
        Ritorna un oggetto con lo stato di un documento in relazione ai
        files presenti che  hanno lo stesso path, ma differiscono nell'estensione
        """
        # {
        #     "path relativo a root dati completo senza estensione" : {
        #         "type": "pdf",
        #         "converted": True,
        #         "processed": False 
        #         "archived": False    
        #     }
        
        # }
        self.dir_content = defaultdict(lambda: {
            "type": None,
            "converted": False,
            "processed": False,
            "archived": False,
        })        
        
        for suffix, files in self.explorer.files.items():
            
            for path in files:
                base = str(path.relative_to(self.explorer.root).with_suffix(""))
                
                if suffix in config.extension:
                    self.dir_content[base]["type"] = suffix[1:]
                elif suffix == ".md":
                    self.dir_content[base]["converted"] = True
                elif suffix == ".json":
                    self.dir_content[base]["processed"] = True
                elif suffix == ".ok":
                    self.dir_content[base]["archived"] = True
        return
                
    def count_converted(self):
        return sum(1 for info in self.dir_content.values() if info["converted"])

    def count_processed(self):
        return sum(1 for info in self.dir_content.values() if info["processed"])

    def count_archived(self):
        return sum(1 for info in self.dir_content.values() if info["archived"])

    def report(self):
        # ---- Reort toekn stimati ed effettivi ---- #
        self.token_consumption()
        self.token_estimation()
        log.info(f"Token stimati per processare l'archivio {self.token_estimation}")
        if not self.real_token_consumption:
            log.info("Nessun dato reale di consumo token disponibile.")
        else:
            log.info("Token realmente usati per processare l'archivio:")
            for field in self.fields:
                if field in self.real_token_consumption:
                    log.info(f"** {field} **")
                    for key in self.keys:
                        value = self.real_token_consumption[field].get(key, 0)
                        log.info(f"    {key}: {value}")
                    log.info("")  

        # ---- Statistiche sui file convertiti/analizzati/archiviati ---- #
        self.document_status()

        total = len(self.dir_content)
        converted = self.count_converted()
        processed = self.count_processed()
        archived = self.count_archived()

        if total == 0:
            log.warning("Nessun file sorgente trovato.")
            return

        log.info("Statistiche sui file:")
        log.info(f"  Totale file unici: {total}")
        log.info(f"  Convertiti: {converted} ({(converted / total) * 100:.2f}%)")
        log.info(f"  Processati: {processed} ({(processed / total) * 100:.2f}%)")
        log.info(f"  Archiviati: {archived} ({(archived / total) * 100:.2f}%)")

        if config.debug:
            log.debug("Elenco file non convertiti:")
            for base, info in self.dir_content.items():
                if not info["converted"]:
                    log.debug(f"  {base}.{info['type']}")


    def save_report_as_json(self):
        report = dict()
        report["tipo_documento"] = "riepilogo"
        report["timestamp"] = datetime.now().isoformat()


        self.document_status()
        total = len(self.dir_content)
        converted = self.count_converted()
        processed = self.count_processed()
        archived = self.count_archived()

        if total > 0:
            report["files"] = {
                "total": total,
                "converted": converted,
                "converted_pct": round((converted / total) * 100, 2),
                "processed": processed,
                "processed_pct": round((processed / total) * 100, 2),
                "archived": archived,
                "archived_pct": round((archived / total) * 100, 2),
            }
        else:
            report["files"] = {
                "total": 0,
                "converted": 0,
                "processed": 0,
                "archived": 0,
            }

        report["token_stimati"] = round(self.token_estimation, 2)

        token_data = {}
        for field in self.fields:
            token_data[field] = {
                key: self.real_token_consumption.get(field, {}).get(key, 0)
                for key in self.keys
            }
        report["token_reali"] = token_data

        stats_file = self.explorer.root / "__gtagger_report.json"
        with open(stats_file, "w") as f:
            json.dump(report, f, indent=2, ensure_ascii=False)

        log.info(f"Report statistico salvato in {stats_file}")


    def save_report_as_csv(self):
        csv_file = Path(self.explorer.root) / "__gtagger_report.csv"
        with open(csv_file, mode='w', newline='', encoding='utf-8') as f:
            writer = csv.writer(f)
            writer.writerow(["path", "type", "converted", "processed", "archived"])
            for base, info in self.dir_content.items():
                writer.writerow([
                    base,
                    info.get("type", ""),
                    info.get("converted", False),
                    info.get("processed", False),
                    info.get("archived", False)
                ])