import hashlib
import logging
import uuid
from pathlib import Path

import chromadb
from chromadb.config import Settings
from chromadb.utils import embedding_functions
from langchain_community.embeddings.fastembed import FastEmbedEmbeddings
from langchain_experimental.text_splitter import SemanticChunker
from langchain_text_splitters import RecursiveCharacterTextSplitter
from tqdm import tqdm

log = logging.getLogger(__name__)


def get_content_hash(text):
    """Generate a consistent hash for document content"""
    return hashlib.md5(text.encode("utf-8")).hexdigest()


class Store:
    def __init__(
        self,
        path="/tmp/chroma",
        collection="default",
        model_name="LaBSE",
        semantic_chunker=False,
        chunk_size=[30,50,70,100,150,200],
        chunk_overlap=[10,20,25,40,50,60],
    ):
        try:
            self.client = chromadb.PersistentClient(
                path=path, settings=Settings(anonymized_telemetry=False)
            )
        except Exception as e:
            log.error(f"Error creating client: {e}")
            return None

        #self.embedding_function = embedding_functions.DefaultEmbeddingFunction()
        self.embedding_function = embedding_functions.SentenceTransformerEmbeddingFunction(model_name=model_name)

        self.collection = self.client.get_or_create_collection(
            name=collection,
            metadata={"hnsw:space": "cosine"},  # or "l2", "ip"
            embedding_function=self.embedding_function,
        )
        log.info(f"Store model={model_name} collection={collection} chunk size={chunk_size} chunk overlap={chunk_overlap}")
        if semantic_chunker:
            log.info(f"Activating semantic chunker %s", model_name)
            embed_model = FastEmbedEmbeddings(model_name=model_name)
            self.text_splitter = SemanticChunker(embed_model, breakpoint_threshold_type="percentile",breakpoint_threshold_amount=0.6)
        else:
            self.text_splitter = [
                RecursiveCharacterTextSplitter(
                    chunk_size=cs,  # number of characters per chunk
                    chunk_overlap=co,  # number of characters to overlap between chunks
                    length_function=len,
                    is_separator_regex=False,
                ) for cs, co in zip(chunk_size, chunk_overlap)
            ]

    def add_file(self, filename, parent_id=""):
        if not parent_id:
            parent_id = str(uuid.uuid4())

        try:
            pfile = Path(filename)
            with pfile.open() as f:
                content = f.read()
                parent_hash = get_content_hash(content)
                # Process content through each text splitter
                all_chunks = []
                for splitter in self.text_splitter:
                    chunks = splitter.split_text(content)
                    all_chunks.append(chunks)

                existing = self.collection.get(
                    where={"parent_hash": {"$eq": parent_hash}},
                )
                if existing["ids"]:
                    log.warning("Duplicate file '%s'", pfile.name)
                    return
                for chunks in all_chunks:
                    for chunk in tqdm(
                        chunks, desc=f"Processing chunks for {pfile.name}", unit="chunk"
                    ):
                        chunk_hash = get_content_hash(chunk)
                        existing = self.collection.get(
                            where={
                                "parent_id,chunk_hash": {"$in": [parent_id, chunk_hash]}
                            },
                        )
                        if existing["ids"]:
                            log.warning("Duplicate chunk on file '%s'", pfile.name)
                            return

                        self.collection.add(
                            documents=[chunk],
                            metadatas=[
                                {
                                    "parent_hash": parent_hash,
                                    "chunk_hash": chunk_hash,
                                    "title": pfile.name,
                                    "path": pfile.parent.name,
                                    "length": len(content),
                                    "parent_id": parent_id,
                                },
                            ],
                            ids=[str(uuid.uuid4())],
                        )
                log.info("Add file '%s'", pfile.name)
        except Exception as e:
            log.error(f"Error adding documents: {e}")

    def search(self, query="", n_results=5, max_metric=1.0):
        result = self.collection.query(
            query_texts=[query],
            n_results=n_results,
            include=["documents", "metadatas", "distances"],
        )
        # from IPython import embed; embed()

        metadatas = result["metadatas"][0]
        documents = result["documents"][0]
        distances = result["distances"][0]
        log.info(f"Distances min: {min(distances)}, max: {max(distances)}, mean: {sum(distances) / len(distances)}")

        # sort metadatas, documents and distances by distance
        metadatas, documents, distances = zip(*sorted(zip(metadatas, documents, distances), key=lambda x: x[2]))

        # Collect match with lowest distance for each parent_id
        response = {}
        for i in range(len(metadatas)):
            if distances[i] <= max_metric :
                parent_id = metadatas[i]["parent_id"]
                if parent_id not in response:
                    response[parent_id] = {
                        "content": documents[i],
                        "meta": metadatas[i],
                        "distance": distances[i],
                        "weighted_score": distances[i],
                        "order": i,
                        "parent_id": parent_id,
                    }
        return [r for r in response.values()]
